#include <iostream>
#include <memory>

#include "point.hpp"

template <typename T> struct Option {
  private:
    T value;
    bool is_some;

  public:
    Option() : is_some(false) {}
    Option(T value) : value(value), is_some(true) {}
};

// Templated alias for unique_ptr
template <typename T> using Box = std::unique_ptr<T>;

template <typename T, typename... Args> Box<T> Enbox(Args &&...args) {
    return std::make_unique<T>(std::forward<Args>(args)...);
}

int main(int argc, char *argv[]) {
    Point p = Point(10, 5);
    std::cout << p << std::endl;
    p.move(5, 5);

    if (p.distance(Point(0, 0)) == p.distanceOrigin()) {
        std::cout << "Distance is the same" << std::endl;
    } else {
        std::cout << "Distance is not the same" << std::endl;
    }

    Box<Point> p_box = std::make_unique<Point>(10, 5);
    Box<Point> p_box2 = Enbox<Point>(10, 5);

    std::cout << p << std::endl;
    std::cout << *p_box << std::endl;
    std::cout << *p_box2 << std::endl;
    std::cout << "Hello, World!" << std::endl;
    return 0;
}
